<?php

//add_action('template_redirect', 'ubudu_redirect_home');

function ubudu_redirect_home() {
	if ( is_home() && !is_paged() ) {
		$args = array(
			'numberposts'     => 1,
			'offset'          => 0,
//			'category'        => ,
			'orderby'         => 'post_date',
			'order'           => 'DESC',
//			'include'         => ,
//			'exclude'         => ,
//			'meta_key'        => ,
//			'meta_value'      => ,
			'post_type'       => 'post',
//			'post_mime_type'  => ,
//			'post_parent'     => ,
			'post_status'     => 'publish'
		);
		$posts_array = get_posts( $args );
		if ( count($posts_array) > 0 ) {
			setup_postdata($posts_array[0]);
			$latest_post_url = get_permalink();
			wp_redirect( $latest_post_url .'#nav-between-posts' );	// defaults to 302
			exit;
		}
	}
}

if(!defined('TEMPLATEPATH')) {
	define( 'TEMPLATEPATH', get_bloginfo('stylesheet_directory'));
}

load_theme_textdomain ('ubudu');

function ubudu_series_init() {
	// create a new taxonomy
	register_taxonomy(
		'series',
		'post',
		array(
			'label' => __( 'Series', 'ubudu' ),
			'sort' => true,
			'args' => array( 'orderby' => 'term_order' ),
			'rewrite' => array(  'slug' => __('series', 'ubudu')  )
		)
	);
}
add_action( 'init', 'ubudu_series_init' );

if ( ! function_exists( 'ubudu_init' ) ) :
function ubudu_init() {

	define('UBUDU_MONTHLY_ARCHIVE_DATE_FORMAT', "F Y");

	if ( function_exists('add_editor_style') ) {
		add_editor_style('editor-style.css.php');
	}

	if ( function_exists('add_theme_support') ) {
		// Add default posts and comments RSS feed links to head
		add_theme_support( 'automatic-feed-links' );
	}

	$GLOBALS['ubudu'] = new Ubudu();
}
endif;

function get_theme_title() {
	return $GLOBALS['ubudu']->theme['Title'];
}
function get_theme_uri() {
	return $GLOBALS['ubudu']->theme['URI'];
}
if ( is_admin() ) {
	switch ( basename($_SERVER['SCRIPT_FILENAME']) ) {
		case 'post.php':
		case 'edit.php':
			add_filter( 'post_link', 'ubudu_post_link' );
			break;

		case 'edit-comments.php':
			load_theme_textdomain ('ubudu');
			add_filter( 'get_comment_link', 'ubudu_comment_link' );
			break;
	}
}
else {
	add_action( 'after_setup_theme', 'ubudu_init' );
}

function ubudu_comment_link ($url) {
	$dash_index = strrpos($url, '-');
	if ( $dash_index === false ) {
		return $url;
	}
	$comment_id = substr($url,$dash_index+1);

	$hash_index = strrpos($url, '#');
	if ( $hash_index === false ) {
		return $url;
	}
	$url_without_hash = substr($url,0,$hash_index);

	return $url_without_hash.'#'.__('comment', 'ubudu').'-'. $comment_id;
}
function ubudu_post_link ($url) {
	$anchor = get_ubudu_post_anchor();

	if ( empty($anchor) ) {
		return $url;
	}
	else {
		return $url.'#'.$anchor;
	}
}

function get_ubudu_post_anchor ( $id = false ) {
	if ( !$id)
		$id = get_the_ID();

	if ( empty ($id) )
		return;

	return sprintf( __('post-%s', 'ubudu'), $id);
}

function get_ubudu_about_url() {
	$about = get_page_by_path( __('about', 'ubudu') );
	if ( !$about ) {
		return false;
	}
	$base_url = get_permalink( $about->ID );
	$anchor = get_ubudu_post_anchor( $about->ID );

	return $base_url . "#" . $anchor;
}

class Ubudu {

	public $theme;
	private $previous_comment_date;

	function __construct() {

		// Hook up theme JavaScript and CSS.
		add_action( 'wp_enqueue_scripts', array($this, 'script') );

		// Work around miscapitalization in translation.
		add_filter( 'get_comment_date', array( $this, 'date_ucfirst' )  );

		// Comment / ping separation based on http://sivel.net/2008/10/wp-27-comment-separation/
		add_filter( 'get_comments_number', array($this, 'comment_count'), 0 );

		// Meta description
		add_action( 'wp_head', array($this, 'meta_description') );

		// Permalink anchors
		add_filter ( 'comment_post_redirect', array($this, 'comment_post_redirect'), 10, 2 );

		// Escape title
		add_filter ( 'the_title', 'esc_attr' );

		// Fix 404 post's title
		add_filter ( 'the_title', array($this, 'title_404') );

		add_filter( 'pre_get_posts', array($this, 'feed_filter'));

		remove_action('wp_head', 'rsd_link');
		remove_action('wp_head', 'wlwmanifest_link');
		remove_action('wp_head', 'index_rel_link');

		// Set up theme data.
		$this->theme = get_theme_data(TEMPLATEPATH.'/style.css');

	}
	function title_404($title) {
		if ( is_404() && in_the_loop() ) {
			$title = ucfirst( $this->get_currently_browsing($tagged = false) );
		}
		return $title;
	}
	function style() {
		wp_enqueue_style('google-web-font-ubuntu', 'http://fonts.googleapis.com/css?family=Ubuntu:400,700,400italic,700italic', array(), '0');
		wp_enqueue_style('jquery-ui', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.14/themes/base/jquery-ui.css', array(), '0' );
		wp_enqueue_style('jquery-contextmenu-style', get_bloginfo('template_url').'/jquery.contextMenu.css', array(), '0');
		wp_enqueue_style('ubudu-style', get_bloginfo('template_url').'/style.css', array(), '0');
		if ($_SERVER['SERVER_NAME'] != 'mummila.net') {
			wp_enqueue_style('paja', get_bloginfo('template_url').'/paja.css', array(), '0');
		}
	}
	function script() {
		wp_enqueue_script("jquery");
		wp_enqueue_script("jquery-cookie", get_bloginfo('template_url').'/js/jquery.cookie.js', array('jquery'), '0');
		wp_enqueue_script('jquery-form');
		wp_enqueue_script('ubudu-acp', get_bloginfo('template_url').'/js/acp.js', array(), '0');
		wp_enqueue_script('acp-i18n', get_bloginfo('template_url').'/js/javascript.php', array(), '0');
		wp_enqueue_script('ubudu-script', get_bloginfo('template_url').'/js/ubudu.js', array('jquery', 'jquery-ui-core'), '0');
		$this->style();
	}
	function get_nav_post_where( $where ) {
		global $wpdb, $post;
		$originalwhere = $where;
		$series = wp_get_object_terms( $post->ID, array('series') );
		if ( count($series) > 0 ) {
			foreach ($series as $serie) {

				$where .= " AND p.ID IN (SELECT $wpdb->term_relationships.object_id FROM `{$wpdb->term_relationships}` WHERE {$wpdb->term_relationships}.term_taxonomy_id = '{$serie->term_taxonomy_id}')";
				return $where;
			}			
		}

		return $where;
	}
	function feed_filter($query) {

		if ( !$query->is_feed || $query->is_category ) {
			return $query;
		}
		$cat = $query->get('cat');
		if ($cat) {
			$comma = ',';
		}
		foreach (
				array(
					__('memes', 'ubudu'),
					__('programming', 'ubudu'),
					__('links', 'ubudu')
			)  as $unwanted ) {
			$cat = get_category_by_path( $unwanted );
			if ($cat) {
				$query->set('cat', $query->get('cat').$comma.'-'.$cat->term_id);
				$comma = ',';
			}
		}
		return $query;
	}

	function meta_description() {
		global $post;

		echo '
	<meta name="description" content="';
		if ( is_singular() ) {

			$tmp_post = $post;
			setup_postdata($post);
			$text = get_the_content();
			if ( empty($post->post_excerpt) ) {
				$text = strip_shortcodes( $text );
			        $text = str_replace(']]>', ']]&gt;', $text);
			        $text = strip_tags($text);
			        $excerpt_length = apply_filters('excerpt_length', 55);
			        $words = preg_split("/[\n\r\t ]+/", $text, $excerpt_length + 1, PREG_SPLIT_NO_EMPTY);
			        if ( count($words) > $excerpt_length ) {
			                array_pop($words);
			                $text = implode(' ', $words);
			                $text = $text . $excerpt_more;
			        } else {
			                $text = implode(' ', $words);
			        }
			}
			else {
				$text = str_replace('<!--:en-->', '', $post->post_excerpt);
				$text = str_replace('<!--:-->', '', $text);
			}
		        $text = str_replace('"', '&quote;', $text);
			echo $text;
			$post = $tmp_post;
			setup_postdata($post);

		}
		else if ( is_home() ) {
			global $paged;
			if ( is_paged() ) {
				printf( __('%1$s, page %2$s', 'ubudu'), get_bloginfo( 'name' ), $paged );
			}
			else {
				printf( __('%1$s, home', 'ubudu'), get_bloginfo( 'name' ) );
			}
		}
		else if ( is_archive() ) {
			printf( __('From the %s archive', 'ubudu'), get_bloginfo( 'name' ) );
		}
		else {

			bloginfo( 'description' );
		}
		echo '" />
';
	}
	function comment_count( $count ) {
	        global $id;

		if ( is_admin() ) {
		        return $count;
		}

	        $comments_by_type = &separate_comments(get_comments('status=approve&post_id=' . $id));
	        return count($comments_by_type['comment']);
	}
	function backreferences_count() {
return "PERSE";
	        global $id;

	        $comments_by_type = &separate_comments(get_comments('status=approve&post_id=' . $id));
	        return count($comments_by_type['pings']);
	}


	public function the_404_post() {
		global $post;
		global $wp_query;
		global $s;
//print __FILE__ . " " . __LINE__;
		$id = $this->get_page_ID_by_slug(__('404-not-found', 'ubudu'));

		if (is_numeric($id)) {
			$post = get_post($id);
			setup_postdata($post);
		}
		else {
			// halp
			die("HALP");
		}
		$wp_query->parse_query();
		$wp_query->is_404 = true;
		$wp_query->is_singular = true;
		if ( $s ) {
			$wp_query->is_search = true;
		}
		else if ( !get_query_var('name') ) {
			$wp_query->is_archive = true;
		}

		return;
	}

	public function search() {
		global $post;
		?>
			<article <?php post_class(  ( is_singular() ? 'single' : '' )  ); ?> id="post-<?php the_ID();?>">
				<?php $this->post_header();
				the_content();
				get_search_form();
				$this->post_footer();
?>			</article><!-- end post-<?php the_ID();?> (the search page) -->
<?php
	}
	public function get_first_category() {
		$classes = get_post_class();
		foreach ($classes as $class) {
			if ( substr($class,0,9) == "category-" && strlen($class) > 9) {
				return $class;
			}
		}
		return false;
	}
	public function post_singular() {
		global $post;
		?>
			<article <?php post_class('single fullpost'); ?> id="post-<?php the_ID();?>">
				<?php
				$this->post_header();
				if ( is_404() ) {
					echo $this->get_suggestions();
				}
				the_content();
				if (  is_page( sanitize_title( ( get_option('category_base') ? get_option('category_base') : 'category') ) )  ) {
					$this->categories_page_categories_list();
				}
				else if (  is_page( sanitize_title( (get_option('tag_base') ? get_option('tag_base') : 'tag') ) )  ) {
					$this->tags_page_tags_list();
				}
				else if ( is_404() ) {
					get_search_form();

				}

				$this->post_footer();
?>
			</article><!-- end post-<?php the_ID();?> -->
			<div class="clear"></div>

<?php
	}
	public function post_header() {

		if ( is_404() ) {
			$id = $this->get_page_ID_by_slug(__('404-not-found', 'ubudu'));
			$title = get_the_title( $id );
			$title_linked = $this->get_currently_browsing($tagged = true);
			$url = 'http'.( $_SERVER["HTTPS"] ? 's' : '' ).'://'.$_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"];
		}
		else {
			$title = get_the_title();
			$title_linked = sprintf( __('%1$s'), '<a
				href="'.get_permalink().'#'.get_ubudu_post_anchor().'"
				title="'.get_the_title().'"
				id="'.get_ubudu_post_anchor().'">'.get_the_title().'</a>'
			);
			$url = get_permalink();
		}

		$id_title = esc_attr(   strip_tags(  sprintf( __('Post no. %d', 'ubudu'), get_the_ID() )  )   );
		$id_permalink = '<a href="'.get_bloginfo('url').'/?p'.(is_page() ? 'age_id' : '').'='.get_the_ID().'#'.sprintf( __('post-number-%d', 'ubudu'), get_the_ID() ).'" title="'.$id_title.'" id="post-permalink-'.get_the_ID().'">#'.get_the_ID().'</a>';
		$date = get_the_time( get_option('date_format') );
		$time = get_the_time( get_option('time_format') );
		$author = get_the_author();
		$author_url = get_the_author_meta('user_url');
		if ($author_url) {
			$author_linked = '<a href="'.$author_url.'" title="'
				.__("Visit the author's homepage", 'ubudu').'">'.get_the_author().'</a>';
			$author = $author_linked;
		}
		$meta = sprintf(
			__('%1$s. %2$s at %3$s, by %4$s.', 'ubudu'),
			$id_permalink,
			ucfirst($date),
			$time,
			$author
		);

		$comments_url = $url.'#'.__('comments', 'ubudu');
		$comments_number = get_comments_number();
		$comments_title = esc_attr( sprintf( __('Comments to %s', 'ubudu'), $title ) );

		$comments_link_text = _n('%d comment.', '%d comments.', $comments_number, 'ubudu');
		$comments_link_text = str_replace( ' ', '&#160;', $comments_link_text );

		$comments_link_single = __('comment.', 'ubudu');
		$comments_link_multiple = __('comments.', 'ubudu');

		$comments_link_text = str_replace( $comments_link_single, '<span class="comments-meta-postfix">'.$comments_link_single.'</span>', $comments_link_text);
		$comments_link_text = str_replace( $comments_link_multiple, '<span class="comments-meta-postfix">'.$comments_link_multiple.'</span>', $comments_link_text);

		$comments_link_text = str_replace( '%d', '<span class="comment-count">%d</span>', $comments_link_text );
		$comments_link_text = sprintf( $comments_link_text, $comments_number );
		if ( is_singular() || is_404() ) {
			$feedback = '<a
							class="response-toggler"
							id="response-toggler-'.get_the_ID().'"
							href="'.$comments_url.'"
							title="'.$comments_title.'">'.$comments_link_text.'</a>
						';
		}
		else {
			$feedback = '<a href="'.$comments_url.'" title="'.$comments_title.'">'.$comments_link_text.'</a>';
		}

		?>
				<header><!-- post header -->
					<h1 id="<?php printf( __('post-number-%d', 'ubudu'), get_the_ID() ); ?>">
						<?php echo $title_linked; ?>
					</h1>
					<h2 class="meta">
						<?php echo $meta; ?>
						<span class="feedback-control"><?php echo $feedback; ?></span>
					</h2>
				</header><!-- end post header -->

				<section class="content">
<?php
	}
	public function post_footer() {
		?>
				</section><!-- end post content -->

				<footer class="clear"><!-- post footer -->
					<?php
		if( get_the_tag_list() ) {
?>

					<?php the_tags('<section class="taxonomy tags alignleft">
'.__('Tags: ', 'ubudu'),',
','

					</section><!-- end tags -->');
?><?php
		}
		else {
?>


					<section class="taxonomy none alignleft">&#160;</section>
<?php
		}
?>
				</footer><!-- end post footer -->
<?php
	}

	public function list_pings($comment, $args, $depth) {
		$GLOBALS['comment'] = $comment;
		?>
		<li <?php comment_class(); ?> id="comment-<?php comment_ID() ?>"><a href="<?php
			echo htmlspecialchars(get_comment_link($comment->comment_ID ) ) ?>"
			title="<?php _e('Permanent link to this pingback', 'ubudu'); ?>"><?php
			printf(__('%1$s at %2$s', 'ubudu'), get_comment_date('j. F\t\a Y'), get_comment_time()) ?></a>
			| <?php printf('<cite class="comment-citation">%s</cite>', get_comment_author_link()) ?>

		<?php if ($comment->comment_approved == '0') {
			echo '<p class="moderated"><small><strong>'.__('This pingback is in moderation.', 'ubudu').'</strong></small></p>';
		}
	}

	public function list_comments($comment, $args, $depth) {

		$GLOBALS['comment'] = $comment;
		?>
				<li <?php comment_class(); ?> id="comment-<?php comment_ID(); ?>">

					<header id="<?php echo __('comment', 'ubudu'); ?>-<?php comment_ID() ?>"><!-- comment header -->
			<?php
		$title = esc_attr(   strip_tags(  sprintf( __('Comment no. %d', 'ubudu'), get_comment_ID() )  )   );
		$permalink = '<a class="comment-permalink-scroller" href="'.get_permalink().'#'.__('comment-number-', 'ubudu').get_comment_ID().'" title="'.$title.'" id="'.__('comment-number-', 'ubudu').get_comment_ID().'">#'.get_comment_ID().'</a>';
		$date = get_comment_time( get_option('date_format') );
		$time = get_comment_time( get_option('time_format') );
		$author = get_comment_author_link();
		echo sprintf(
				__('%1$s. %2$s at %3$s, by %4$s.', 'ubudu'),
				$permalink,
				ucfirst($date),
				$time,
				$author
		);
		?>
					</header><!-- end comment header -->

					<section class="content clear">
						<div class="comment-author alignleft"><?php echo get_avatar($comment,$size='96',$default='',$alt=get_comment_author()); ?></div>
						<div class="comment-text"><?php comment_text(); ?></div>
					</section>

					<footer><!-- comment footer -->
						<?php if ($comment->comment_approved == '0') : ?>
						<p class="is-in-moderation"><small><strong><?php _e('This comment is in moderation.', 'ubudu'); ?></strong></small></p>
						<?php endif; ?>
					</footer><!-- end comment footer -->
		<?php
	}

	public function list_main_comments($comment, $args, $depth) {

		$GLOBALS['comment'] = $comment;
		?>
			<div class="comment"><aside <?php comment_class('main-comment'); ?> id="comment-<?php comment_ID(); ?>">
				<header>
					<h2><?php
		$title = esc_attr(   strip_tags(  sprintf( __('%$1s: %2$s', 'ubudu'), get_comment_author(), $this->get_comment_attribute(get_comment_text()) )  )   );
		$permalink = '<a href="'.get_permalink().'#'.__('comment', 'ubudu').'-'.get_comment_ID().'" title="'.$title.'" class="comment-permalink-scroller" id="main-comment-permalink-'.get_comment_ID().'">#'.get_comment_ID().'</a>';
		$date = get_comment_time( get_option('date_format') );
		$time = get_comment_time( get_option('time_format') );
		$author = '<strong>'.get_comment_author_link().'</strong>';
		echo sprintf(
			__('%1$s. %2$s at %3$s, by %4$s.', 'ubudu'),
			$permalink,
			ucfirst($date),
			$time,
			$author
		);
		?></h2>
				</header>
				<section class="content clear"><?php comment_text() ?></section>

				<footer><!-- comment footer -->
					<?php if ($comment->comment_approved == '0') : ?>
					<p class="is-in-moderation"><small><strong><?php _e('This comment is in moderation.', 'ubudu'); ?></strong></small></p>
					<?php endif; ?>
				</footer><!-- end comment footer -->

			</aside><!-- end comment-<?php comment_ID();?> -->
<?php
	}
	function get_post_by_path($page_path, $output = OBJECT, $post_type = 'page') {
		return get_page_by_path($page_path, $output, 'post');
	}

	// http://erikt.tumblr.com/post/278953342/get-a-wordpress-page-id-with-the-slug
	function get_page_ID_by_slug($slug) {
		$page = get_page_by_path($slug);
		if ($page) {
			return $page->ID;
		} else {
			return null;
		}
	}
	public function comment_post_redirect ($link, $comment) {
		if ( empty($link) ) {
			return $link;
		}

		$comment_anchor = sprintf( __('comment-%s', 'ubudu'), $comment->comment_ID );
		if (strpos($link, '#') !== false) {
			$link = substr($link,0,strpos($link, '#'));
		}
		return $link.'#' . $comment_anchor;
	}
	public function get_currently_browsing($tagged = true) {
		global $wp_query;
		global $s;
		global $post;

		$url = 'http'.( $_SERVER["HTTPS"] ? 's' : '' ).'://'.$_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"];

		if ( !get_query_var('name') && !$s ) {
			$wp_query->is_archive = true;

		}
		$currently = '';
		if ( is_search() ) {
			$searched_for = $s;
			$searched_for = trim(  esc_html( (urldecode($searched_for)) )  );
			$searched_for = ( empty($searched_for) ? __('nothing', 'ubudu') : $searched_for );
			$currently = $searched_for;
		}
		else if ( is_404() && !is_archive() ) {
			$searched_for = get_query_var('name');
			$searched_for = str_replace('-', ' ', $searched_for);
			$searched_for = str_replace('+', ' ', $searched_for);
			$searched_for = trim(  esc_html( (urldecode($searched_for)) )  );
			$searched_for = ( empty($searched_for) ? __('nothing', 'ubudu') : $searched_for );
			$currently = esc_html( ucfirst($searched_for) );
		}
		else if ( is_category() ) {
			$what = single_cat_title('', false);
			if ( $what == esc_attr( __('Memes', 'ubudu') )  ) {
				$what = esc_attr( __('Memes', 'ubudu') );
			}
			else if (  $what == esc_attr( __('Links', 'ubudu') )  ) {
				$what = esc_attr( __('Links', 'ubudu') );
			}
			else if (  $what == esc_attr( __('Pictures', 'ubudu') )  ) {
				$what = esc_attr( __('Picture posts', 'ubudu') );
			}
			else if (  $what == esc_attr( __('Programming', 'ubudu') )  ) {
				$what = esc_attr( __('Regular programming', 'ubudu') );
			}
			else if (  $what == esc_attr( __('Entries', 'ubudu') )  ) {
				$what = esc_attr( __('Diary entries', 'ubudu') );
			}
			else {
				$what = 'wat';
			}

			if ( $what == 'wat' ) {
				$what = single_cat_title('', false);
			}

			$currently = $what;
		}
		else if ( is_tag() ) {
			$currently = sprintf(
				__('Posts tagged %1$s', 'ubudu'),
				single_tag_title('', false)
			);
		}
		else if (  is_tax( 'series' )  ) {
			$currently = sprintf(
				__('Posts in series %1$s', 'ubudu'),
				single_tag_title('', false)
			);
		}
		else if ( is_author() ) {
			global $wp_query;
			$curauth = $wp_query->get_queried_object();
			if ( empty($curauth) ) {
				$nick = $_GET['author'];
			}
			$currently = sprintf(
				__('Posts by %1$s', 'ubudu'),
				$curauth->display_name
			);
		}
		else if ( is_time() ) {
			global $wp_locale;

			$d = get_query_var('day');
			$m = get_query_var('monthnum');
			$y = get_query_var('year');

			if ( $d ) {
				$m = $wp_locale->month[ sprintf('%02d', $m) ];
				$date = sprintf( __('%1$s. %2$s %3$s', 'ubudu'), $d, $m, $y );
			}
			else if ( $m ) {
				$m = $wp_locale->month[ sprintf('%02d', $m) ];
				$date = ucfirst( sprintf( __('%1$s %2$s', 'ubudu'), $m, $y ) );
			}
			else if ( $y ) {
				$date = $y;
			}

			$currently = __('The archive', 'ubudu');
			if ( $date ) {
				$currently = sprintf(
					__('The archive from %1$s', 'ubudu'),
					$date
				);
			}
		}
		else if ( is_404() ) {
			global $wp_locale;

			$d = get_query_var('day');
			$m = get_query_var('monthnum');
			$y = get_query_var('year');

			if ( $d ) {
				$m = $wp_locale->month[ sprintf('%02d', $m) ];
				$date = sprintf( __('%1$s. %2$s %3$s', 'ubudu'), $d, $m, $y );
			}
			else if ( $m ) {
				$m = $wp_locale->month[ sprintf('%02d', $m) ];
				$date = ucfirst( sprintf( __('%1$s %2$s', 'ubudu'), $m, $y ) );
			}
			else if ( $y ) {
				$date = $y;
			}

			$currently = __('The archive', 'ubudu');
			if ( $date ) {
				$currently = sprintf(
					__('The archive from %1$s', 'ubudu'),
					$date
				);
			}
		}
		else if ( is_home() ) {
			$currently = sprintf(
				__('The latest posts', 'ubudu')
			);
		}

		if ( is_paged() ) {
			$page = $wp_query->get( 'paged' );
			$currently = sprintf(
				__('%1$s, page %2$s', 'ubudu'),
				$currently,
				$page
			);
		}

		$currently = esc_html($currently);
		if ($tagged) {
			return '<a href="'.$url.'" title="'.esc_attr($currently).'">'.$currently.'</a>';
		}
		else {
			return $currently;
		}
	}
	function get_next_post_datetime_archive() {
		if ( is_year() ) {

			$current_year = (int)get_the_time('Y');
			$next_year = $current_year + 1;
			$previous_post_datetime = sprintf('%04d-01-01 00:00:00', $next_year);
		}
		else if ( is_month() ) {
			$current_year = (int)get_the_time('Y');
			$current_month = (int)get_the_time('m');
			if ($current_month >= 12) {
				$next_year = $current_year + 1;
				$next_month = 1;
				$previous_post_datetime = sprintf('%04d-%02d-01 00:00:00', $next_year, $next_month);
			}
			else {
				$next_month = $current_month + 1;
				$previous_post_datetime = sprintf('%04d-%02d-01 00:00:00', $current_year, $next_month);
			}
		}
		else if ( is_day() ) {
			$current_year = (int)get_the_time('Y');
			$current_month = (int)get_the_time('m');
			$days_this_month = cal_days_in_month(CAL_GREGORIAN, $current_month, $current_year);
			$current_day = (int)get_the_time('d');
			if ($current_day >= $days_this_month) {

				$next_day = 1;
			}
			else {
				$next_day = $current_day + 1;
				$next_month = $current_month + 1;
			}
			if ($next_month >= 12) {
				$next_month = 1;
				$next_year = $current_year + 1;
				$previous_post_datetime = sprintf('%04d-%02d-%02d 00:00:00', $next_year, $next_month, $next_day);
			}
			else {
				$previous_post_datetime = sprintf('%04d-%02d-%02d 00:00:00', $current_year, $current_month, $next_day);
			}
		}

		return $previous_post_datetime;
	}

	function get_next_post_datetime() {
		global $wpdb;

		$previous_datetime = get_the_time('Y-m-d H:i:s');

		$q = "SELECT ".$wpdb->prefix."posts.post_date FROM $wpdb->posts WHERE ".$wpdb->prefix."posts.post_status = 'publish' AND ".$wpdb->prefix."posts.post_date > '".$previous_datetime."' ORDER BY ".$wpdb->prefix."posts.post_date ASC LIMIT 0,1";

		$post = $wpdb->get_row($q, OBJECT);

		return $post->post_date;
	}			

	function get_kuta_content_attribute($content) {
		$kuta_max_post_length = 140;
		$content_stripped = strip_tags( $content );
		if ( strlen ($content_stripped) > $kuta_max_post_length ) {
			$content = substr($content_stripped, 0, $kuta_max_post_length-1).'&hellip;';
		}
		else {
			$content = $content_stripped;
		}
		$colon = ( ctype_upper(substr($content,0,1)) ? ': ' : '' );
		$colon = ( empty($colon) ? ( substr($content,0,1) == ',' ? '' : ' ' ) : $colon );
		return esc_attr( strip_tags( get_the_author().$colon.$content) );
	}
	function get_comment_attribute($content) {
		$kuta_max_post_length = 140;
		$content_stripped = strip_tags( $content );
		if ( strlen ($content_stripped) > $kuta_max_post_length ) {
			$content = substr($content_stripped, 0, $kuta_max_post_length-1).'&hellip;';
		}
		else {
			$content = $content_stripped;
		}
		return esc_attr( strip_tags( $content) );
	}
	function kuta_content ($content) {
		$colon = ( ctype_upper(substr($content,0,1)) ? '<span class="segmental-colon">:</span> ' : '' );
		$colon = ( empty($colon) ? ( substr($content,0,1) == ',' ? '' : ' ' ) : $colon );
		return '<span class="author">'.get_the_author().'</span>'.$colon.$content;
	}


	// Mimicked this from http://svn.automattic.com/wordpress/trunk/wp-includes/canonical.php
	function get_suggestions() {
		global $wpdb;
		global $s;

		// Store for re-use later.
		$name = get_query_var('name');
		if ( empty($name) )
			$name = $s;

		if ( empty($name) )
			return false;

		// if any of year, monthnum, or day are set, use them to refine the query
		if ( get_query_var('year') )
			$where_date = $wpdb->prepare(" AND YEAR(post_date) = %d", get_query_var('year'));
		if ( get_query_var('monthnum') )
			$where_date = $wpdb->prepare(" AND MONTH(post_date) = %d", get_query_var('monthnum'));
		if ( get_query_var('day') )
			$where_date = $wpdb->prepare(" AND DAYOFMONTH(post_date) = %d", get_query_var('day'));

		$where = $wpdb->prepare("post_name LIKE %s", $name . '%') .$where_date;
		$post_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE $where AND post_status = 'publish'");

		if ( !$post_id ) {
			$where = $wpdb->prepare("post_name LIKE %s", '%' . $name) .$where_date;
			$post_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE $where AND post_status = 'publish'");
		}
		if ( !$post_id ) {
			$where = $wpdb->prepare("post_name LIKE %s", '%' . $name . '%') .$where_date;
			$post_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE $where AND post_status = 'publish'");
		}
		if ( !$post_id ) {
			$where = $wpdb->prepare("post_title LIKE %s", $name . '%') .$where_date;
			$post_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE $where AND post_status = 'publish'");
		}
		if ( !$post_id ) {
			$where = $wpdb->prepare("post_title LIKE %s", '%' . $name) .$where_date;
			$post_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE $where AND post_status = 'publish'");
		}
		if ( !$post_id ) {
			$where = $wpdb->prepare("post_title LIKE %s", '%' . $name . '%') .$where_date;
			$post_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE $where AND post_status = 'publish'");
		}

		if ( !$post_id ) {
			return false;
		}
		return '<p class="ubudu-suggestions">'
				.sprintf(__('Did you mean: %s', 'ubudu'),
					'<a href="'.get_permalink($post_id).'#'.get_ubudu_post_anchor($post_id)
					.'" title="'.get_the_title($post_id).'"><strong>'
					.get_the_title($post_id).'</strong></a>')
			.'</p>';
	}

	public function date_ucfirst($date) {
		return ucfirst(strtolower($date));
	}

	public function tags_page_tags_list() {
		$show_em_like_this = array(
		'smallest'  => 6, 
		'largest'   => 42,
		'unit'      => 'pt', 
		'number'    => 0,  
		'format'    => 'list',
		'separator' => '
	',
		'orderby'   => 'name', 
		'order'     => 'ASC',
//		'exclude'   => ,
//		'include'   => ,
		'link'      => 'view', 
		'taxonomy'  => 'post_tag', 
		'echo'      => true
		);

		wp_tag_cloud($show_em_like_this);
	}

	public function categories_page_categories_list() {
		$show_em_like_this = array(
//			'show_option_all'    => ,
			'orderby'            => 'name',
			'order'              => 'ASC',
			'show_last_update'   => 0,
			'style'              => 'list',
			'show_count'         => 1,
			'hide_empty'         => 1,
			'use_desc_for_title' => 1,
			'child_of'           => 0,
			'feed'               => ' <small><small>•)</small></small>) ',
//			'feed_type'          => ,
//			'feed_image'         => get_bloginfo('template_url').'/images/rss-12.png',
//			'exclude'            => ,
//			'exclude_tree'       => ,
//			'include'            => ,
			'current_category'   => 0,
			'hierarchical'       => true,
			'title_li'           => '',
			'number'             => NULL,
			'echo'               => 1,
			'depth'              => 0
		);

		echo '<ul class="all-categories">';
		wp_list_categories($show_em_like_this);
		echo '</ul>';
	}
	public function nav_bot() {
		if ( is_single() ) {
			$this->nav_older_post();
		}
		else {
			$this->nav_older_posts();
		}
	}
	public function nav_top() {
		if ( is_single() ) {
			$this->nav_newer_post();
		}
		else {
			$this->nav_newer_posts();
		}
	}
	private function nav_older_posts() {
		?>
	<nav class="between-pages older">
		<?php next_posts_link( '&#171; '.__('Older posts', 'ubudu') ); ?>
	</nav>
		<?php
	}
	private function nav_newer_posts() {
		?>
	<nav class="between-pages newer">
&#160;
		<?php previous_posts_link( __('Newer posts', 'ubudu').' &#187;' ); ?>
	</nav>
		<?php
	}
	private function nav_older_post() {
		?>
	<nav class="between-posts older">
		<?php previous_post_link( __('&#171; %link', 'ubudu') ); ?>
	</nav>
		<?php
	}
	private function nav_newer_post() {
		?>
	<nav class="between-posts newer">
&#160;
		<?php next_post_link( __('%link &#187;', 'ubudu') ); ?>
	</nav>
		<?php
	}
	public function get_paged_pages() {
		global $wp_query;
		global $paged;
		$paged_pages = '';
		if ($wp_query->max_num_pages > 1) {
			$page_base = 'page';
			if ($paged > 1) {
				$cp = $paged;
				$pat = '@'.$paged.'/$@';
				$paged_uri_base = preg_replace($pat, '', $_SERVER["REQUEST_URI"]);
				$unpat = '@/'.$page_base.'/'.$paged.'/$@';
				$unpaged_uri_base = preg_replace($unpat, '/', $_SERVER["REQUEST_URI"]);
			}
			else {
				$cp = 1;
				$unpaged_uri_base = $_SERVER["REQUEST_URI"];
				$paged_uri_base = $unpaged_uri_base . $page_base . '/';
			}

			$pages = array();
			$paging_limit = 5;	// keep it odd!
			$prefix = '';
			$trailer = '';
			if ($wp_query->max_num_pages > $paging_limit) {
				if ( $cp-($paging_limit / 2) < 1 ) {
					$start_from = 1;
				}
				else {
					if ($wp_query->max_num_pages - $cp-floor($paging_limit / 2) < $paging_limit) {
						$start_from = $wp_query->max_num_pages - $paging_limit +1;
					}
					else {
						$start_from = $cp-floor($paging_limit / 2);
					}

					if ($start_from > 1) {
						$prefix = '<a href="'.$unpaged_uri_base.'" title="'
							.esc_attr(  sprintf( __('Page 1 of %1$d', 'ubudu'), $wp_query->max_num_pages )  ).'">1</a>&hellip;';
					}
				}
				if ( $cp+($paging_limit / 2) > $wp_query->max_num_pages ) {
					$go_until = $wp_query->max_num_pages;
				}
				else {
					if ( $cp+($paging_limit / 2) < $paging_limit ) {
						$go_until = $paging_limit;
					}
					else {
						$go_until = floor( $cp+($paging_limit / 2) );
					}

					if ($go_until < $wp_query->max_num_pages) {
						$trailer = '&hellip;<a href="'.$paged_uri_base.$wp_query->max_num_pages.'/" title="'
							.esc_attr(  sprintf( __('Page %1$d of %1$d', 'ubudu'), $wp_query->max_num_pages )  ).'">'.$wp_query->max_num_pages.'</a>';
					}
				}
			}
			else {
				$start_from = 1;
				$go_until = $wp_query->max_num_pages;
			}

			if ($start_from == 1) {
				$pages[1] = '<a href="'.$unpaged_uri_base.'" title="'
						.sprintf( __('Page 1 of %1$d', 'ubudu'), $wp_query->max_num_pages ).'">1</a>';
				$start_from = 2;
			}
			for ($i = $start_from; $i <= $go_until; $i++) {
				$pages[$i] = '<a href="'.$paged_uri_base.$i.'/" title="'
					.sprintf( __('Page %1$d of %2$d', 'ubudu'), $i, $wp_query->max_num_pages ).'">'.$i.'</a>';
			}
			$pages[$cp] = '<span title="'.sprintf( __('Page %1$d of %2$d', 'ubudu'), $cp, $wp_query->max_num_pages )
					.'" class="current-paged-page">'.$cp.'</span>';
			$paged_pages = $prefix. implode(' ', $pages) .$trailer;
		}
		return $paged_pages;
	}
}
?>
