// http://hughevans.net/2008/05/28/jquery-toggler
// https://beski.wordpress.com/2009/04/21/scroll-effect-with-local-anchors-jquery/
// http://michael.theirwinfamily.net/articles/jquery/collapsible-fieldsets-jquery-plugin-drupal-style

// helper function for often-used scrolling.
jQuery.fn.scrollTo = function() {
	var target_offset = jQuery(this).offset();
	if (target_offset === undefined) {
		return false;
	}
	return jQuery('html, body').animate({scrollTop:target_offset.top}, 500);
}

var permalinkSlider = function(event) {

	// only react to lmb.
	if (event.which != '1') {
		return;
	}

	// bail out right away if scrolling won't work.
	if ( !jQuery(this).scrollTo() ) {
		return false;
	}

	// hash needs to be sliced from href, because (at the very least)
	// paragraph permalinks' id's aren't where those links are pointing to.
	var hash;
	if ( jQuery(this).attr('href') !== undefined ) {
		var href = jQuery(this).attr('href');
		var hashIndex = href.lastIndexOf('#');
		if (hashIndex >= 0) {
			hash = href.substr(hashIndex);
		}
	}

	// if no href (why?), fall back to id if available.
	if ( hash === undefined ) {
		if ( jQuery(this).attr('id') !== undefined ) {
			hash = '#'+jQuery(this).attr('id');
		}
	}

	if ( hash !== undefined ) {
		window.location.hash = '#'+jQuery(this).attr('id');
		event.preventDefault();
	}

	// if no id, fall back to browser default.
	return;
}

jQuery(document).ready(function() {

	// turn post, comment and paragraph permalinks to sliding links.
	jQuery('article header a[rel="bookmark"]').click(permalinkSlider);
	jQuery('a.paragraph-permalink').click(permalinkSlider);
	jQuery('a.comment-permalink-scroller').click(permalinkSlider);

	// collapsibles' low-level functionality.
	jQuery.fn.collapse = function() {
		var options = {
			delay : (arguments[0] || 250),
			callback : (arguments[1] || function() { return false; })
		}
		var collapsible = jQuery(this);
		return jQuery.when(collapsible.children().not('#response-prompt').each(function() {
			jQuery(this).hide(options.delay, function() {
				jQuery(this).css('display', 'none');	// TODO: isn't this redundant?
			}); // end hide() callback
		})).done( (collapsible.addClass('collapsed') && options.callback()) ); // end each() callback
	}
	jQuery.fn.uncollapse = function() {
		var options = {
			delay : (arguments[0] || 250),
			callback : (arguments[1] || function() { return false; })
		}
		var collapsible = jQuery(this);
		return jQuery.when(collapsible.children().not('#response-prompt').each(function() {
			jQuery(this).show(options.delay);
		})).done( ( collapsible.removeClass('collapsed') && options.callback()) ); // end each() callback
	}
	jQuery.fn.toggleCollapse = function() {
		var options = {
			delay : (arguments[0] || 250),
			callback : (arguments[1] || function() { return false; })
		}
		var collapsible = jQuery(this);
		if ( collapsible.hasClass('collapsed') ) {
			return collapsible.uncollapse(options.delay, options.callback);
		}
		else {
			return collapsible.collapse(options.delay, options.callback);
		}
	}

	// how the collapsibles' toggling widget works.
	jQuery('.collapser').click(function(event) {

		// only react to lmb.
		if (event.which != '1') {
			return;
		}

		// improve legibility further down.
		var collapser = jQuery(this);
		var collapsible = jQuery(this).parents('.collapsible');

		collapsible.toggleCollapse(250, function() {
			var target;
			var hash;

			// the form has been closed.
			if ( collapsible.hasClass('collapsed') ) {

				// if no comments have been entered,
				// hide comments altogether again.
				if ( jQuery('.response').children('.comments').find('li').length == 1 ) {
					jQuery('.response-hider').trigger({
						type: 'click',
						which: 1
					});
				}

				// otherwise go to hider at top.
				target = jQuery('.response-hider');
				hash = comments_i18n_hash;
			}

			// the form's open, so go there.
			else {
				target = collapser;
				hash = comment_prompt_i18n_hash;
				if ( collapsible.find('input:visible:enabled:first').length == 0 ) {
					collapsible.find('textarea:visible:enabled:first').focus();
				}
				else {
					collapsible.find('input:visible:enabled:first').focus();
				}
			}

			target.scrollTo();
			window.location.hash = hash;
			event.preventDefault();
		});
	});

	// how comments popping up and down works.
	jQuery.fn.popUp = function(callback) {

		var response = jQuery(this);

		// comments are hidden, so fade the background and use
		// comments link for the popup's positioning.
		jQuery(".popup-background").fadeIn("slow");
		var toggleroffset = jQuery('.response-toggler').offset();
		response.css({
			'top': toggleroffset.top+50+'px',
			'left': toggleroffset.left-320+'px'
		});

		response.show(250, function() {

			// readily expand the form if the post 
			// has no comments.
			// note that there's always at least one
			// li element, the placeholder for new comments.
			if ( response.children('.comments').find('li').length == 1 ) {
				jQuery(".collapsible").uncollapse();
			}
			if ( response.find('input:visible:enabled:first').length == 0 ) {
				response.find('textarea:visible:enabled:first').focus();
			}
			else {
				response.find('input:visible:enabled:first').focus();
			}

			return callback();
		});
	}
	jQuery.fn.popDown = function(callback) {
		jQuery(this).hide(250, function() {

			// collapse the form inside the now collapsed comments section.
			jQuery(".collapsible").collapse();

			// fade content in = fade background out.
			jQuery(".popup-background").fadeOut("slow");
			return callback();
		});
	}

	// bind popup functionality to the comments link.
	jQuery('.response-toggler').click(function(event) {

		// only react to lmb.
		if (event.which != '1') {
			return;
		}

		// ease legibility further down.
		var toggler = jQuery(this);
		var response = jQuery('.response');

		// pop the comments.
		response.popUp(function() {
			var hash, target;

			// if the form's hidden, scroll to
			// comments closing widget.
			var collapsible = jQuery(".collapsible");
			if ( collapsible.hasClass('collapsed') ) {
				target = jQuery('.response-hider');
				hash = comments_i18n_hash;
			}

			// the form's open, scroll to it.
			else {
				target = jQuery('.collapser');
				hash = comment_prompt_i18n_hash;
			}

			if (target != undefined) {
				target.scrollTo();
			}
			if (hash != undefined) {
				window.location.hash = hash;
				event.preventDefault();
			}
		});
	});

	// now that we've reached here and things work, set things up.

	// collapse 'collapsed' elements on load.
	jQuery('.collapsed').collapse();

	// hide togglable elements on load.
	jQuery('.togglable').hide();

	// reveal bottom nav since comments are pop-up.
	jQuery('nav.between.posts').show();

	// faded background.
	jQuery('.popup-background').css({
		'display': 'none',
		'opacity': '0.7', 
		'background': 'black',
		'position': 'fixed',
		'width': '100%',
		'height': '100%',
		'top': '0',
		'left': '0',
		'z-index': '2'
	});

	// turn comments section into a popup.
	jQuery('.response').css({
		'border-radius': '20px',
		'background': '#111',
		'position': 'absolute',
		'padding': '20px',
		'width': '640px',
		'top': '20px',
		'left': '20px',
		'z-index': '10'
	});

	// inject popup closing widget.
	jQuery('.response').prepend('<img src="'+xit+'" width="18" height="18" class="response-hider" alt="'+xit_alt+'" title="'+xit_popup_title+'" />');

	// bind popdown functionality to widget.
	jQuery('.response-hider').click(function(event) {

		// only react to lmb.
		if (event.which != '1') {
			return;
		}

		// ease legibility further down.
		var toggler = jQuery(this);
		var response = jQuery('.response');

		// pop the comments.
		response.popDown(function() {
			var hash, target;

			// find the post title element and its id.
			if ( response.attr('id') !== undefined ) {

				var id = response.attr('id');
				var lastDash = id.lastIndexOf('-');
				if ( lastDash >= 0 && lastDash+1 < id.length ) {
					var postNumber = id.substring(lastDash+1);
					if (Math.floor(postNumber) == postNumber) {
						var posthash = post_number_i18n_hash + postNumber;
						target = jQuery(posthash);
						hash = posthash;
					}
				}
			}

			if (target != undefined) {
				target.scrollTo();
			}
			if (hash != undefined) {
				window.location.hash = hash;
				event.preventDefault();
			}
		});
	});

	// make the widget act active upon hovering.
	jQuery('.response-hider').hover(
		function(event) {
			jQuery(this).css('cursor','pointer');
		},
		function (event) {
			jQuery(this).css('cursor','auto');
		}
	);

	// pick search form from the footer, not for example from the 404 post
	var searchForm = jQuery('footer #searchform');

	// insert default search terms just before it's all taken away.
	// scripted just so that it won't clog the input if JS is disabled.
	var searchInput = searchForm.children('input');
	searchInput.attr( 'value', search_terms_i18n.toLowerCase() );

	// apropos: make the default disappear when clicked.
	searchInput.click(function(event) {

		// only react to lmb.
		if (event.which != '1') {
			return;
		}

		// have we been moved to the menu?
		var defaultTerms = search_terms_i18n;
		if ( searchForm.parents('.menu-item').length > 0 ) {
			defaultTerms = menu_search_terms_i18n.toLowerCase();
		}
		if ( searchInput.attr('value') == defaultTerms ) {
			searchInput.attr('value', '');
		}
		searchInput.focus();
	});
	searchInput.focusout(function(event) {

		// have we been moved to the menu?
		var defaultTerms = search_terms_i18n;
		if ( searchForm.parents('.menu-item').length > 0 ) {
			defaultTerms = menu_search_terms_i18n.toLowerCase();
		}
		if ( jQuery.trim(searchInput.attr('value')) == '' ) {
			searchInput.attr('value', defaultTerms);
		}
	});

	jQuery.fn.menu = function() {
		var menuTrigger = jQuery(this);
		var menuContainer = menuTrigger.parents('.menu-container');

		var menuItems = {
			"menu_home": {
				"id": menu_home_i18n,
				"title": menu_home_title_i18n.toLowerCase()
			},
			"menu_search": {
				"id": menu_search_i18n,
				"title": menu_search_title_i18n.toLowerCase()
			},
			"menu_archive": {
				"id": menu_archive_i18n,
				"title": menu_archive_title_i18n.toLowerCase()
			},
			"menu_about": {
				"id": menu_about_i18n,
				"title": menu_about_title_i18n.toLowerCase()
			}
		};

		var i = 0;
		var menuHtml = '<ul id="menu">';
		for (item in menuItems) {
			var item = menuItems[item];
			i++;
			menuHtml += '<li class="menu-item menu-item-'+i+'" id="'+item["id"]+'"><a class="menu-item-trigger menu-'+item["id"]+'-trigger" href="#'+item["id"]+'" title="'+item["title"]+'">'+item["title"]+'</a></li>';
		}
		menuHtml += '</ul>';
		var menu = jQuery(menuHtml).appendTo(menuContainer);

		// set the home link.
		var homeItem = menuItems["menu_home"];
		var homeLink = jQuery('#'+homeItem["id"]).children('a').attr('href', home_url);

		// bring search into the menu.
		var searchItem = menuItems["menu_search"];
		var menuSearchLi = jQuery('#'+searchItem["id"]);
		menuSearchLi.empty();
		searchForm.appendTo(menuSearchLi);

		searchForm.children('input').attr('value', menu_search_terms_i18n.toLowerCase()).css('width', '7em');
		searchForm.children('input').addClass('menu-'+menu_search_i18n+'-trigger');
		searchForm.children('button[type="submit"]').hide();

		// localization means we don't know a priori which of the items is widest.
		var previousItem;
		for (i in menuItems) {
			var item = menuItems[i];
			var widest = jQuery('#'+item["id"]);
			if ( previousItem === undefined || previousItem.width() < widest.width() ) {
				for (i in menuItems) {
					var item = menuItems[i];
					jQuery('#'+item["id"]).css( 'width', widest.width() );
				}
			}
			widest.children('*').css('width', '100%');
			previousItem = widest;
		}
		menu.hide();

		// http://stackoverflow.com/a/1670561/717501
		var menuFadeOut = function() {
			menu.fadeOut("slow", function() {

				// only the menu search.
				if ( searchForm.parents('.menu-item').length > 0 ) {

					// reset keywords.
					searchInput.attr('value', menu_search_terms_i18n.toLowerCase());

					// focus on the first title.
					jQuery('article:first header h1 a').focus();
				}
			});
		};
		var menuTimerStart = function(event) {
			menu.data( 'timeout', setTimeout(menuFadeOut, 650) );
		};
		var menuTimerStop = function(event) {
			clearTimeout( menu.data('timeout') );
		};
		var menuShow = function(event) {
			menuTimerStop();
			menu.fadeIn("slow");
		};

		var menuTimerEnable = function(event) {
			menuContainer.mouseenter(menuShow).mouseleave(menuTimerStart);
		};

		var menuTimerDisable = function(event) {
			menuTimerStop();
			menuContainer.unbind('mouseenter', menuShow).unbind('mouseleave', menuTimerStart);
		};

		menuTimerEnable();
		menuTrigger.mousemove(
			function(event) {
				var newTop = event.pageY - Math.floor( menu.height() / 2 );
				if ( newTop < 0 ) newTop = 0;
				menu.css('top', newTop);
			}
		);

		// width for menu popups.
		var popsWidth = jQuery('#ubudu').width() - menuSearchLi.width();
		popsWidth = Math.floor(.8 * popsWidth);

		// how menu popups work.
		var menupop = function menupop(event) {

			// only react to lmb.
			if (event.which != '1') {
				return;
			}

			var popup = jQuery(this).parents('li').children('.popup');
			event.preventDefault();
			popup.toggle('slow', function() {
				if ( popup.is(':visible') ) {
					jQuery(".popup-background").fadeIn("slow");
					menuTimerDisable();
					jQuery('.menu-popup-trigger').unbind('click', menupop);
					jQuery(document).one('click', function unpop(event) {

						// only react to lmb.
						if (event.which != '1') {
							jQuery(document).one('click', unpop());
							return;
						}

						var clicked = jQuery(event.target);
						if (								// pop down when the user clicks something that...
						   ( !clicked.is('a') && clicked.parents('a').length < 1 )	// is not a link
						|| ( clicked.is('a') && clicked.parents('.popup').length < 1 )  // is a link, but not inside the popup
						) {
							jQuery(".popup-background").fadeOut("slow");
							event.preventDefault();
							popup.hide('slow', function() {
								menuTimerEnable();
								jQuery('.menu-popup-trigger').click(menupop);
								if ( !menuContainer.is(':hover') ) {
									menuTrigger.trigger('mouseleave');
								}
							});
						}
						// let the link do its default thing (e.g. no preventDefault()) if it's inside the popup.
					});
				}
				// this should never happen, but it shouldn't be a problem if it did, either.
				else {
					menuTimerEnable();
					jQuery('.menu-popup-trigger').click(menupop);
					jQuery(".popup-background").fadeOut("slow");
				}
			});
		};

		// bring archive listing into the menu.
		var archive = jQuery('nav.archives');
		archive.addClass('popup');
		var archiveItem = menuItems["menu_archive"];
		var archiveLi = jQuery( '#'+archiveItem["id"] );
		var archiveLink = archiveLi.children('a');
		archiveLink.attr('href', archive_url);
		if ( archive.length > 0 && archiveLi.length > 0 ) {
			archive.appendTo(archiveLi).hide();
			archive.css( 'width', popsWidth );
			archiveLink.addClass('menu-popup-trigger');
			archiveLink.click(menupop);
		}

		// bring colophon into the menu.
		var colophon = jQuery('#colophon');
		var aboutItem = menuItems["menu_about"];
		var aboutLi = jQuery( '#'+aboutItem["id"] );
		var aboutLink = aboutLi.children('a');
		aboutLink.attr('href', about_url);
		if ( colophon.length > 0 && aboutLi.length > 0 ) {
			var colophonwrapper = jQuery('<div class="popup">').appendTo(aboutLi).hide();
			colophon.appendTo(colophonwrapper);
			colophon.css( 'width', popsWidth );
			aboutLink.addClass('menu-popup-trigger');
			aboutLink.click(menupop);
		}
	};
	jQuery('.menu-trigger').menu();



	// apply hash.
	var hash = window.location.hash;
	if(hash) {

		// hash calls for the comment form to be open.
		if ( hash == comment_prompt_hash || hash == comment_prompt_i18n_hash ) {
			var response = jQuery('.response');
			response.popUp(function() {
				var collapser = jQuery('.collapser');
				var collapsible = jQuery('.collapsible');
				if ( collapsible.hasClass('collapsed') ) {
					return collapser.trigger({
						type: 'click',
						which: 1
					});
				}
				else {
					collapser.scrollTo();
					return;
				}
			});
		}

		// hash calls for comments to be open.
		else if (
			hash == comments_hash || hash == comments_i18n_hash
			|| hash == references_hash || hash == references_i18n_hash
			|| hash.indexOf(comment_hash) == 0 || hash.indexOf(comment_i18n_hash) == 0
			|| hash.indexOf(comment_number_hash) == 0 || hash.indexOf(comment_number_i18n_hash) == 0
		) {
			var response = jQuery('.response');
			response.popUp(function() {
				var target;

				// scroll to hashed element.
				if (jQuery(hash).length > 0) {
					target = jQuery(hash);
				}

				// fallback: scroll to hider.
				else {
					target = jQuery('.response-hider');
				}

				// don't return so as to fall back to default.
				target.scrollTo();
			});
		}
		// hash calls for something in the menu.
		else if (
			   hash == menu_search_hash_i18n
			|| hash == menu_archive_hash_i18n
			||  hash == menu_about_hash_i18n
		) {
			var menuTrigger = jQuery('.menu-trigger');
			menuTrigger.scrollTo();
			menuTrigger.trigger('mouseenter');

			var targetId = hash.substring(1);
			var target = '.'+'menu-'+targetId+'-trigger';
			jQuery(target).trigger({
				type: 'click',
				which: 1
			});
		}
	}

	jQuery('<span class="beta">Beta!</span>').appendTo(jQuery('.menu-trigger'));
});

